// Generated by CoffeeScript 1.9.3
var JsonML, cheerio, htmlBeautify, htmlparser2;

htmlparser2 = require('cheerio/node_modules/htmlparser2');

cheerio = require('cheerio');

htmlBeautify = require('js-beautify').html;

JsonML = (function() {
  function JsonML() {}

  JsonML.prototype.stringifyListMode = true;

  JsonML.prototype.stringify = function(object, replacer, indent) {
    var element, html, j, len;
    html = '';
    if (this.stringifyListMode) {
      for (j = 0, len = object.length; j < len; j++) {
        element = object[j];
        html += this.stringifyElement(element, replacer);
      }
    } else {
      html += this.stringifyElement(object, replacer);
    }
    if (indent > 0) {
      html = htmlBeautify(html, {
        indent_size: indent,
        unformatted: ['code', 'pre', 'em', 'strong', 'span']
      });
      html = html.replace(/^\s*/g, '').replace(/(\r\n|\n){2,}/g, '\n');
    }
    return html;
  };

  JsonML.prototype.stringifyElement = function(element, replacer) {
    var attributes, elementList, i, j, len, list, name, node, ref;
    if (typeof element === 'string') {
      node = element;
    } else {
      if (typeof element[0] !== 'string') {
        throw new TypeError('Invalid tagName "' + element[0] + '"');
      }
      name = element.shift();
      if (((ref = element[0]) != null ? ref.toString() : void 0) === '[object Object]') {
        attributes = element.shift();
      }
      elementList = element != null ? element : [];
      node = cheerio('<' + name + '/>');
      if (attributes != null) {
        node.attr(attributes);
      }
      for (i = j = 0, len = elementList.length; j < len; i = ++j) {
        list = elementList[i];
        node.append(this.stringifyElement(list));
      }
    }
    if (replacer != null) {
      node = replacer(node);
    }
    return node;
  };

  JsonML.prototype.parse = function(html, trim) {
    var nodes, object;
    if (trim == null) {
      trim = true;
    }
    nodes = htmlparser2.parseDOM(html, {
      xmlMode: true
    });
    object = this.parseElementList(nodes, trim);
    return object;
  };

  JsonML.prototype.parseElementList = function(nodes, trim) {
    var canConcat, element, elementList, i, j, len, node;
    if (trim == null) {
      trim = true;
    }
    i = -1;
    elementList = [];
    for (j = 0, len = nodes.length; j < len; j++) {
      node = nodes[j];
      element = this.parseElement(node, trim);
      if (typeof element === 'string') {
        if (trim) {
          element = element.trim();
        }
        if (element === '&nbsp;' && trim) {
          element = '';
        }
      }
      if ((element != null ? element.length : void 0) === 0) {
        continue;
      }
      canConcat = typeof elementList[i] === 'string' && typeof element === 'string';
      if (canConcat) {
        elementList[i] += element;
      } else {
        if (element != null) {
          elementList.push(element);
        }
        i++;
      }
    }
    return elementList;
  };

  JsonML.prototype.parseElement = function(node, trim) {
    var attribs, child, children, data, element, elementList, j, len, name, type;
    if (trim == null) {
      trim = true;
    }
    type = node.type, data = node.data, name = node.name, attribs = node.attribs, children = node.children;
    switch (type) {
      case 'directive':
        return '<' + data + '>';
      case 'comment':
        return '<!--' + data + '-->';
      case 'text':
        return data;
      case 'tag':
      case 'script':
        elementList = this.parseElementList(children, trim);
        element = [];
        if (name != null) {
          element.push(name);
        }
        if (Object.keys(attribs).length) {
          element.push(attribs);
        }
        for (j = 0, len = elementList.length; j < len; j++) {
          child = elementList[j];
          element.push(child);
        }
        return element;
      default:
        throw new TypeError(type + ' is Invalid node type');
    }
  };

  return JsonML;

})();

module.exports = new JsonML;

module.exports.JsonML = JsonML;
